####################################################################
# Example LASSO code for Survey Practice special issue
#
# Example of how one might conduct ALASSO and Logistic regression
# for the survey response example.  This is not the full replication 
# code for our analysis.  It is only meant to illustrate the concepts.  
# If one’s computer has at least four cores, we recommend running the 
# polywog commands using the .parallel=T option.  To run these 
# commands without parallelization, delete the six lines just after 
# “# to run in parallel” and omit the .parallel=T argument in 
# the polywog commands.
#
# Curtis S. Signorino, Antje Kirchner, 10/27/17
####################################################################
rm(list=ls())
library(polywog)
library(caret)					# needed for confusionMatrix()

####################################################################
# to run in parallel
####################################################################
library(parallel) 						
library(foreach)						
library(doParallel)						
threads=detectCores()					
cl=makeCluster(threads-1)				
registerDoParallel(cl)					

####################################################################
# load data.  create test and training data sets.
####################################################################
setwd('~/Dropbox/propscores/SVM/replication')  # change to your path
load("SPData2.Rda")
SPData2=transform(SPData2, newrespond20 = ifelse(newrespond20=="Respondent",1,0))
dtrain=subset(SPData2,fold!=11)				# training data
dtest=subset(SPData2,fold==11)				# test data

####################################################################
# formula:  response variable ~ covariates
####################################################################
f = newrespond20 ~ age + sex  + wborace + hispanic2 + region + educ3 + incgrp4 + telstat + wrkcata  + ratcat2

####################################################################
# adaptive lasso with polynomial expansion.
# use 10-fold cross validation to find degree and lambda
####################################################################
cv.poly = cv.polywog(f, data=dtrain, family="binomial", method="alasso", .parallel=TRUE, nfolds=10) 
cv.poly										# print results
m.poly = cv.poly$polywog.fit					# best model

p.poly = predict(m.poly, newdata=dtest, type='response')		# predicted probs
p.poly.01 = ifelse(p.poly>=.5, 1, 0)		# predicted outcomes
confusionMatrix(p.poly.01, dtest$newrespond20, positive="1")

####################################################################
# logistic regression
####################################################################
m.logit = glm(f, data=dtrain, family=binomial(logit))
p.logit = predict(m.logit, newdata=dtest, type='response')	# predicted probs
p.logit.01 = ifelse(p.logit>=.5, 1, 0)		# predicted outcomes
confusionMatrix(p.logit.01, dtest$newrespond20, positive="1")

stopCluster(cl)							 


